@@ -5,23 +5,60 @@ import numpy as np
 import matplotlib.pyplot as plt
 
 import astropy.stats
+import astropy.units as u
 
 CLIGHT = 299792458.0 # m/s
 
+TRUE_VALUES = [True, 1, '1', 'True', 'TRUE', 'true', 'y', 'yes', 'Y', 'Yes']
+FALSE_VALUES = [False, 0, '0', 'False', 'FALSE', 'false', 'n', 'no', 'N', 'No']
+
+FNU_CGS = u.erg/u.second/u.cm**2/u.Hz
+FLAM_CGS = u.erg/u.second/u.cm**2/u.Angstrom
+
+def bool_param(value, false_values=FALSE_VALUES, true_values=TRUE_VALUES, which='false', check_both=True):
+    """
+    Flexible booleans
+    
+    If ``which == 'false'``, test that ``value not in false_values``.
+    
+    If ``which == 'true'``, test ``value in true_values``.
+    
+    If ``check_both`` and ``value`` isn't in either list, return the value 
+    itself.
+    
+    """
+    if which == 'false':
+        test = value not in false_values
+    elif which == 'true':
+        test = value in true_values
+    else:
+        raise ValueError("Option ``which`` must be 'true' or 'false'")
+    
+    if check_both:
+        if value not in true_values + false_values:
+            test = value
+            
+    return test
+
+
 def path_to_eazy_data():
+    """
+    Return internal path to ``eazy/data``.
+    """
     return os.path.join(os.path.dirname(__file__), 'data')
 
+
 def set_warnings(numpy_level='ignore', astropy_level='ignore'):
     """
     Set global numpy and astropy warnings
     
     Parameters
     ----------
-    numpy_level : {'ignore', 'warn', 'raise', 'call', 'print', 'log'}
-        Numpy error level (see `~numpy.seterr`).
+    numpy_level : 'ignore', 'warn', 'raise', 'call', 'print', 'log'
+        Numpy error level (see `numpy.seterr`).
         
-    astropy_level : {'error', 'ignore', 'always', 'default', 'module', 'once'}
-        Astropy error level (see `~warnings.simplefilter`).
+    astropy_level : 'error', 'ignore', 'always', 'default', 'module', 'once'
+        Astropy error level (see `warnings.simplefilter`).
     
     """
     from astropy.utils.exceptions import AstropyWarning
@@ -93,9 +130,16 @@ def running_median(xi, yi, NBIN=10, use_median=True, use_nmad=True, reverse=Fals
     return xm, ym, ys, N
 
 def nmad(arr):
+    """
+    Normalized median absolute deviation statistic
+    
+    Includes the normalization factor of 1.48, see 
+    `astropy.stats.median_absolute_deviation`.
+    """
     import astropy.stats
     return 1.48*astropy.stats.median_absolute_deviation(arr)
 
+
 def log_zgrid(zr=[0.7,3.4], dz=0.01):
     """Make a logarithmically spaced redshift grid
     
@@ -115,7 +159,30 @@ def log_zgrid(zr=[0.7,3.4], dz=0.01):
     """
     zgrid = np.exp(np.arange(np.log(1+zr[0]), np.log(1+zr[1]), dz))-1
     return zgrid
+
+
+def trapz_dx(x):
+    """
+    Return trapezoid rule coefficients, useful for numerical integration 
+    using a dot product
     
+    Parameters
+    ----------
+    x : array-like
+        Independent variable
+    
+    Returns
+    -------
+    dx : array_like
+        Coefficients for trapezoidal rule integration.
+    """
+    dx = np.zeros_like(x)
+    diff = np.diff(x)/2.
+    dx[:-1] += diff
+    dx[1:] += diff
+    return dx
+
+
 def clipLog(im, lexp=1000, cmap=[-1.4914, 0.6273], scale=[-0.1,10]):
     """
     Return normalized array like DS9 log
@@ -128,6 +195,7 @@ def clipLog(im, lexp=1000, cmap=[-1.4914, 0.6273], scale=[-0.1,10]):
     
     return clip_log
 
+
 def get_mw_dust(ra, dec, **kwargs):
     """
     Wrapper around functions to try to query for the MW E(B-V)
@@ -178,7 +246,7 @@ def get_irsa_dust(ra=53.1227, dec=-27.805089, type='SandF'):
     ra, dec : float
         RA/Dec in decimal degrees.
         
-    type : 'SFD' or 'SandF'
+    type : 'SFD', 'SandF'
         Dust model, with        
             SandF = Schlafly & Finkbeiner 2011 (ApJ 737, 103) 
               SFD = Schlegel et al. 1998 (ApJ 500, 525)
@@ -208,10 +276,11 @@ def get_irsa_dust(ra=53.1227, dec=-27.805089, type='SandF'):
         return float(str(stats.refPixelValueSFD).split()[0])
     else:
         return float(str(stats.refPixelValueSandF).split()[0])
-        
+
+
 def fill_between_steps(x, y, z, ax=None, *args, **kwargs):
     """
-    Make `fill_between` work like linestyle='steps-mid'.
+    Make `matplotlib.pyplot.fill_between` work like linestyle='steps-mid'.
     """
     so = np.argsort(x)
     mid = x[so][:-1] + np.diff(x[so])/2.
@@ -225,10 +294,24 @@ def fill_between_steps(x, y, z, ax=None, *args, **kwargs):
     
     ax.fill_between(xfull[so], yfull[so], zfull[so], *args, **kwargs)
 
+
+def safe_invert(arr):
+    """
+    Version-safe matrix inversion using `numpy.linalg.inv` or `numpy.matrix.I`
+    """
+    try:
+        from numpy.linalg import inv
+        _inv = inv(arr)
+    except:
+        _inv = np.matrix(arr).I.A
+    
+    return _inv
+
+
 class GalacticExtinction(object):
     def __init__(self, EBV=0, Rv=3.1, force=None, radec=None, ebv_type='SandF'):
         """
-        Wrapper to use either `~specutils.extinction` or the `~extinction` 
+        Wrapper to use either `specutils.extinction` or the `extinction` 
         modules, which have different calling formats.  The results from 
         both of these modules should be equivalent.
                 
@@ -246,9 +329,10 @@ class GalacticExtinction(object):
             
         force : None, 'extinction', 'specutils.extinction'
             Force use one or the other modules.  If `None`, then first try
-            to import `~specutils.extinction` and if that fails use
-            `~extinction`.
+            to import `specutils.extinction` and if that fails use
+            `extinction`.
         """
+        import importlib
         
         # Import handler
         if force == 'specutils.extinction':
@@ -257,13 +341,33 @@ class GalacticExtinction(object):
         elif force == 'extinction':
             from extinction import Fitzpatrick99
             self.module = 'extinction'
+        elif force == 'dust_extinction':
+            from dust_extinction.parameter_averages import F99
+            self.module = 'dust_extinction'
         else:
-            try:
-                import specutils.extinction
-                self.module = 'specutils.extinction'
-            except:
-                from extinction import Fitzpatrick99
-                self.module = 'extinction'
+            modules = [['dust_extinction.parameter_averages', 'F99'], 
+                       ['extinction','Fitzpatrick99'],
+                       ['specutils.extinction','ExtinctionF99']]
+            
+            self.module = None
+            for (mod, cla) in modules:
+                try:
+                    _F99 = getattr(importlib.import_module(mod), cla)
+                    self.module = mod
+                    break
+                except:
+                    continue
+            
+            if self.module is None:
+                raise ImportError("Couldn't import extinction module from "
+                                  "dust_extinction, extinction or specutils") 
+                                       
+            # try:
+            #     from specutils.extinction import ExtinctionF99
+            #     self.module = 'specutils.extinction'
+            # except:
+            #     from extinction import Fitzpatrick99
+            #     self.module = 'extinction'
         
         if radec is not None:
             self.EBV = get_mw_dust(ra=radec[0], dec=radec[1], type=ebv_type)
@@ -271,17 +375,26 @@ class GalacticExtinction(object):
             self.EBV = EBV
         
         self.Rv = Rv
-        self.Av = EBV*Rv
         
-        if self.module == 'specutils.extinction':
-            self.f99 = specutils.extinction.ExtinctionF99(self.Av)
+        if self.module == 'dust_extinction.parameter_averages':
+            self.f99 = _F99(Rv=self.Rv)
+            
+        elif self.module == 'specutils.extinction':
+            self.f99 = _F99(self.Av)
             #self.Alambda = f99(self.wave*u.angstrom)
         else:
-            self.f99 = Fitzpatrick99(self.Rv)
+            self.f99 = _F99(self.Rv)
             #self.Alambda = f99(self.wave*u.angstrom, Av)
     
+    @property
+    def Av(self):
+        return self.EBV*self.Rv
+    
+    @property    
     def info(self):
-        print('F99 extinction with `{0}`: Rv={1:.1f}, E(B-V)={2:.3f} (Av={3:.2f})'.format(self.module, self.Rv, self.EBV, self.Av))
+        msg = ('F99 extinction with `{0}`: Rv={1:.1f}, '
+              'E(B-V)={2:.3f} (Av={3:.2f})')
+        return msg.format(self.module, self.Rv, self.EBV, self.Av)
         
     def __call__(self, wave):
         """
@@ -289,13 +402,13 @@ class GalacticExtinction(object):
         
         Parameters
         ----------
-        wave : float or `~numpy.ndarray`
+        wave : float or `numpy.ndarray`
             Observed-frame wavelengths.  If no `unit` attribute available, 
-            assume units are `~astropy.units.Angstrom`.
+            assume units are `astropy.units.Angstrom`.
         
         Returns
         -------
-        Alambda : like `wave`
+        Alambda : like ``wave``
             F99 extinction (mags) as a function of wavelength.  Output will
             be set to zero below 909 Angstroms and above 6 microns as the
             extinction modules themselves don't compute outside that range.
@@ -303,25 +416,66 @@ class GalacticExtinction(object):
         """
         import astropy.units as u
         if not hasattr(wave, 'unit'):
-            unit = u.angstrom
+            unit = u.Angstrom
         else:
             unit = 1
-                
+                       
         inwave = np.squeeze(wave).flatten()
-        clip = (inwave*unit > 909*u.angstrom) & (inwave*unit < 6*u.micron)
-        Alambda = inwave*0.
-        
+        if self.module == 'dust_extinction.parameter_averages':
+            clip = (inwave*unit > 1/10.*u.micron) 
+            clip &= (inwave*unit < 1/0.3*u.micron)
+        else:
+            clip = (inwave*unit > 909*u.angstrom) & (inwave*unit < 6*u.micron)
+        Alambda = np.zeros(inwave.shape)
+                
         if clip.sum() == 0:
             return Alambda
         else:
-            if self.module == 'specutils.extinction':
+            if self.module == 'dust_extinction.parameter_averages':
+                flam = self.f99.extinguish(inwave[clip]*unit, Av=self.Av)
+                Alambda[clip] = -2.5*np.log10(flam)
+                
+            elif self.module == 'specutils.extinction':
                 Alambda[clip] = self.f99(inwave[clip]*unit)
             else:
                 Alambda[clip] = self.f99(inwave[clip]*unit, self.Av)
         
         return Alambda
 
-def zphot_zspec(zphot, zspec, zlimits=None, zmin=0, zmax=4, axes=None, figsize=[6,7], minor=0.5, skip=2, selection=None, catastrophic_limit=0.15, title=None, min_zphot=0.02, alpha=0.2, extra_xlabel='', extra_ylabel='', xlabel=r'$z_\mathrm{spec}$', ylabel=r'$z_\mathrm{phot}$', **kwargs):
+def abs_mag_to_luminosity(absmag, pivot=None, output_unit=u.L_sun):
+    """
+    Convert absolute AB mag to luminosity units
+    
+    Parameters
+    ----------
+    absmag : array-like
+        Absolute AB magnitude.
+    
+    pivot : float
+        Filter pivot wavelength associated with the magnitude.  If no units, 
+        then assume `astropy.units.Angstrom`.
+    
+    output_unit : `astropy.units.core.Unit`
+        Desired output unit.  Must specify a ``pivot`` wavelength for output
+        power units, e.g., `astropy.unit.L_sun`.
+    
+    """
+    if pivot is None:
+        nu = 1.
+    else:
+        if hasattr(pivot, 'unit'):
+            wunit = 1
+        else:
+            wunit = u.Angstrom
+            
+        nu = ((CLIGHT*u.m/u.second)/(pivot*wunit)).to(u.Hz)
+        
+    fjy = 3631*u.jansky * 10**(-0.4*absmag)
+    d10 = (10*u.pc).to(u.cm)
+    f10 = fjy * 4 * np.pi * d10**2 * nu
+    return f10.to(output_unit)
+    
+def zphot_zspec(zphot, zspec, zlimits=None, zmin=0, zmax=4, axes=None, figsize=[6,7], minor=0.5, skip=2, selection=None, catastrophic_limit=0.15, title=None, min_zphot=0.02, alpha=0.2, extra_xlabel='', extra_ylabel='', xlabel=r'$z_\mathrm{spec}$', ylabel=r'$z_\mathrm{phot}$', label_pos=(0.05, 0.95), label_kwargs=dict(ha='left', va='top', fontsize=10), label_prefix='', format_axes=True, color='k', point_label=None, **kwargs):
     """
     Make zphot_zspec plot scaled by log(1+z) and show uncertainties
     """
@@ -342,12 +496,14 @@ def zphot_zspec(zphot, zspec, zlimits=None, zmin=0, zmax=4, axes=None, figsize=[
     NOUT = (clip & ~clip_cat).sum()
     
     gs = GridSpec(2,1, height_ratios=[6,1])
-    if axes is None:
+    NEW_AXES = axes is None
+    if NEW_AXES:
         fig = plt.figure(figsize=figsize)
         ax = fig.add_subplot(gs[0,0])
     else:
         ax = axes[0]
-
+        fig = None
+        
     if title is not None:
         ax.set_title(title)
 
@@ -361,7 +517,7 @@ def zphot_zspec(zphot, zspec, zlimits=None, zmin=0, zmax=4, axes=None, figsize=[
         ax.errorbar(np.log10(1+zspec[clip & clip_cat]), 
                     np.log10(1+zphot[clip & clip_cat]), 
                     yerr=yerr[:,clip & clip_cat], marker='.', alpha=alpha, 
-                    color='k', linestyle='None')
+                    color=color, linestyle='None', label=point_label)
     else:
         ax.scatter(np.log10(1+zspec[clip & ~clip_cat]),
                    np.log10(1+zphot[clip & ~clip_cat]), 
@@ -369,37 +525,55 @@ def zphot_zspec(zphot, zspec, zlimits=None, zmin=0, zmax=4, axes=None, figsize=[
 
         ax.scatter(np.log10(1+zspec[clip & clip_cat]), 
                    np.log10(1+zphot[clip & clip_cat]), 
-                   marker='.', alpha=alpha, color='k')
+                   marker='.', alpha=alpha, color=color, label=point_label)
         
-    xt = np.arange(zmin, zmax+0.1, minor)
-    xl = np.log10(1+xt)
-    ax.plot(xl, xl, color='r', alpha=0.5)
-    ax.set_xlim(xl[0], xl[-1]); ax.set_ylim(xl[0],xl[-1])
-    xtl = list(xt)
+    if NEW_AXES | format_axes:
+        xt = np.arange(zmin, zmax+0.1, minor)
+        xl = np.log10(1+xt)
+        ax.plot(xl, xl, color='r', alpha=0.5)
+        ax.set_xlim(xl[0], xl[-1])
+        ax.set_ylim(xl[0],xl[-1])
+        xtl = list(xt)
 
-    if skip > 0:
-        for i in range(1, len(xt), skip):
-            xtl[i] = ''
+        if skip > 0:
+            for i in range(1, len(xt), skip):
+                xtl[i] = ''
 
-    ax.set_xticks(xl); ax.set_xticklabels([]);
-    ax.set_yticks(xl); ax.set_yticklabels(xtl);
-    ax.grid()
-    ax.set_ylabel(ylabel + extra_ylabel)
+        ax.set_xticks(xl)
+        if axes is None:
+            ax.set_xticklabels([])
+        else:
+            if len(axes) == 1:
+                ax.set_xticks(xl)
+                ax.set_xticklabels(xtl);
+                ax.set_xlabel(xlabel + extra_xlabel)
+            
+        ax.set_yticks(xl); ax.set_yticklabels(xtl);
+        ax.set_ylabel(ylabel + extra_ylabel)
 
     sample_nmad = nmad(dz[clip])
     sample_cat_nmad = nmad(dz[clip & clip_cat])
 
-    ax.text(0.05, 0.925, r'N={0} ({4}, {1:4.1f}%), $\sigma$={2:.4f} ({3:.4f})'.format(clip.sum(), frac_cat*100, sample_nmad, sample_cat_nmad, NOUT),
-            ha='left', va='top', fontsize=10, transform=ax.transAxes)
+    if label_pos is not None:
+        msg = r'{label_prefix} N={N} ({NOUT}, {err_frac:4.1f}%), $\sigma$={sample_nmad:.4f} ({sample_cat_nmad:.4f})'
+        msg = msg.format(label_prefix=label_prefix, 
+                         N=clip.sum(), err_frac=frac_cat*100, 
+                         sample_nmad=sample_nmad, 
+                         sample_cat_nmad=sample_cat_nmad, NOUT=NOUT)
+                         
+        ax.text(label_pos[0], label_pos[1], msg, transform=ax.transAxes)
     
 
     if axes is None:
         ax = fig.add_subplot(gs[1,0])
     else:
-        ax = axes[1]
-    
+        if len(axes) == 2:
+            ax = axes[1]
+        else:
+            return True
+            
     if zlimits is not None:
-        yerr = np.abs(zlimits.T-zphot)#/(1+self.cat['z_spec'])
+        yerr = np.abs(zlimits.T-zphot)
         ax.errorbar(np.log10(1+zspec[clip & ~clip_cat]), dz[clip & ~clip_cat], 
                     yerr=yerr[:,clip & ~clip_cat], 
                     marker='.', alpha=alpha, color='r', linestyle='None')
@@ -413,29 +587,132 @@ def zphot_zspec(zphot, zspec, zlimits=None, zmin=0, zmax=4, axes=None, figsize=[
         ax.scatter(np.log10(1+zspec[clip & clip_cat]), dz[clip & clip_cat],
                     marker='.', alpha=alpha, color='k')
         
-    ax.set_xticks(xl); ax.set_xticklabels(xtl);
-    ax.set_xlim(xl[0], xl[-1])
-    ax.set_ylim(-6*sample_nmad, 6*sample_nmad)
-    ax.set_yticks([-3*sample_nmad, 0, 3*sample_nmad])
-    ax.set_yticklabels([r'$-3\sigma$',r'$0$',r'$+3\sigma$'])
-    ax.set_xlabel(xlabel + extra_xlabel)
-    ax.set_ylabel(r'$\Delta z / 1+z$')
-    ax.grid()
+    if fig is not None:
+        ax.set_xticks(xl); ax.set_xticklabels(xtl);
+        ax.set_xlim(xl[0], xl[-1])
+        ax.set_ylim(-6*sample_nmad, 6*sample_nmad)
+        ax.set_yticks([-3*sample_nmad, 0, 3*sample_nmad])
+        ax.set_yticklabels([r'$-3\sigma$',r'$0$',r'$+3\sigma$'])
+        ax.set_xlabel(xlabel + extra_xlabel)
+        ax.set_ylabel(r'$\Delta z / 1+z$')
+        for a in fig.axes:
+            a.grid()
 
-    fig.tight_layout(pad=0.1)
+        fig.tight_layout(pad=0.1)
+        return fig
+    else:
+        return True
+
+
+def query_html(ra, dec, **kwargs):
+    """
+    Return HTML string of queries at a position
+    """
+    html = f"({ra:.6f}, {dec:.6f}) "
+    for func, name in zip([cds_query, eso_query, mast_query, alma_query, show_legacysurvey, hscmap_query], ['CDS','ESO','MAST','ALMA', 'LEG','HSC']):
+        url = func(ra, dec, **kwargs)
+        html += f' <a href="{url}">{name}</a>'
     
-    return fig
+    return html
+    
+def cds_query(ra, dec, radius=1., **kwargs):
+    """
+    Open browswer with CDS catalog query around central position
+    """
+    #rd = self.get('pan fk5').strip()
+    rd = f'{ra} {dec}'
+    rdst = rd.replace('+', '%2B').replace('-', '%2D').replace(' ', '+')
+    url = (f'http://vizier.u-strasbg.fr/viz-bin/VizieR?'
+           f'-c={rdst}&-c.rs={radius:.1f}')
+           
+    #os.system(f'open {url}')
+    return url
+
+def eso_query(ra, dec, radius=1., dp_types=['CUBE','IMAGE'], extra='', **kwargs):
+    """
+    Open browser with ESO archive query around central position.
+    
+    ``radius`` in arcmin.
+    """
+    #ra, dec = self.get('pan fk5').strip().split()
+    
+    dp_type = ','.join(dp_types)
+    
+    url = (f'https://archive.eso.org/scienceportal/home?'
+            f'pos={ra},{dec}&r={radius/60.}&dp_type={dp_type}{extra}')
+                    
+    #os.system(f'open {url}')
+    return url
+
+def mast_query(ra, dec, instruments=['WFC3','ACS','WFPC2'], max=1000, **kwargs):
+    """
+    Open browser with MAST archive query around central position
+    """
+    #ra, dec = self.get('pan fk5').strip().split()
+    if len(instruments) > 0:
+        instr='&sci_instrume='+','.join(instruments)
+    else:
+        instr = ''
         
+    url = (f'https://archive.stsci.edu/hst/search.php?RA={ra}&DEC={dec}'
+           f'&sci_aec=S{instr}&max_records={max}&outputformat=HTML_Table'
+            '&action=Search')
+            
+    #os.system(f'open {url}')
+    return url
+
+def alma_query(ra, dec, mirror="almascience.eso.org", radius=1, extra='', **kwargs):
+    """
+    Open browser with ALMA archive query around central position
+    """
+    #ra, dec = self.get('pan fk5').strip().split()
+
+    url = (f"https://{mirror}/asax/?result_view=observation"
+           f"&raDec={ra}%20{dec},{radius}{extra}")
+    #os.system(f'open "{url}"')
+    return url
+
+
+def hscmap_query(ra, dec, open=True, **kwargs):
+    """
+    Function to open HSC explorer in browser centered on target coordinates
+    """
+    
+    import os
+    rrad = ra/180*np.pi
+    drad = dec/180*np.pi
+    url = (f"https://hsc-release.mtk.nao.ac.jp/hscMap-pdr2/app/#/?_=%7B%22view%22%3A%7B%22a%22%3A{rrad},%22d%22%3A{drad}"
+           ",%22fovy%22%3A0.00009647627785850188,%22roll%22%3A0%7D,%22sspParams%22%3A%7B%22type%22%3A%22"
+           "SDSS_TRUE_COLOR%22,%22filter%22%3A%5B%22HSC-Y%22,%22HSC-Z%22,%22HSC-I%22%5D,%22simpleRgb"
+           "%22%3A%7B%22beta%22%3A22026.465794806718,%22a%22%3A1,%22bias%22%3A0.05,%22b0%22%3A0%7D,%22"
+           "sdssTrueColor%22%3A%7B%22beta%22%3A40106.59228119989,%22a%22%3A2.594451857120983,%22bias%22%3A0.05,"
+           "%22b0%22%3A0%7D%7D,%22externalTiles%22%3A%5B%5D,%22activeReruns%22%3A%5B%22pdr2_wide%22,%22pdr2_dud"
+           "%22%5D%7D")
+        
+    return url
+
+
+def show_legacysurvey(ra, dec, layer='dr8', zoom=17, **kwargs):
+    """
+    Open browser with legacysurvey.org panner around central position
+    """
+    #ra, dec = self.get('pan fk5').strip().split()
+    url = (f'http://legacysurvey.org/viewer?ra={ra}&dec={dec}'
+           f'&layer={layer}&zoom={zoom}')
+            
+    #os.system(f'open {url}')
+    return url
+    
 def interp_conserve(x, xp, fp, left=0., right=0.):
     """
-    Interpolation analogous to `~numpy.interp` but conserving "flux".
+    Interpolation analogous to `numpy.interp` but conserving "flux".
     
     Parameters
     ----------
-    x : `~numpy.ndarray`
+    x : `numpy.ndarray`
         Desired interpolation locations
 
-    xp, fp : `~numpy.ndarray`
+    xp, fp : `numpy.ndarray`
         The `x` and `y` coordinates of the function to be interpolated.  The
         `x` array can be irregularly spaced but should be increase
         monotonically.
@@ -450,10 +727,10 @@ def interp_conserve(x, xp, fp, left=0., right=0.):
         Interpolated values.
     
     Interpolation performed by trapezoidal integration between the midpoints
-    of the output `x` array with `~numpy.trapz`.
+    of the output `x` array with `numpy.trapz`.
     
     .. note:: For a faster `cython` implementation of this function, see 
-              `~grizli.utils_c.interp_conserve_c`.
+              `grizli.utils_c.interp_conserve_c`.
               
     """
     midpoint = (x[1:]-x[:-1])/2.+x[:-1]
@@ -629,6 +906,8 @@ class emceeChain():
         """
         Make a FITS file of an EMCEE chain
         """
+        import astropy.io.fits as pyfits
+        
         header = pyfits.Header()
         header.update('NWALKERS', self.nwalkers)
         header.update('NSTEP', self.nstep)
@@ -647,6 +926,11 @@ class emceeChain():
             print('Wrote %s.' %(file))
     
     def load_fits(self, file='emcee_chain.fits'):
+        """
+        Load emcee chain fits file created by ``save_fits``.
+        """
+        import astropy.io.fits as pyfits
+        
         im = pyfits.open(file)
         self.nwalkers = im[0].header['NWALKERS']
         self.nstep = im[0].header['NSTEP']
@@ -664,7 +948,8 @@ class emceeChain():
             show = self.param_names
         
         NP = len(show)
-        fig = unicorn.plotting.plot_init(square=True, aspect=1, xs=size, left=0.05, right=0.01, bottom=0.01, top=0.01, NO_GUI=False, use_tex=False, fontsize=7)
+        #fig = unicorn.plotting.plot_init(square=True, aspect=1, xs=size, left=0.05, right=0.01, bottom=0.01, top=0.01, NO_GUI=False, use_tex=False, fontsize=7)
+        fig = plt.figure(figsize=(7,7))
         fig.subplots_adjust(wspace=0.0,hspace=0.0)
         
         counter = 0
